#!/usr/bin/env bash
#
# Read memory usage in Linux
set -euo pipefail
IFS=$'\n\t'
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

SLEEP_INTERVAL=${SLEEP_INTERVAL:-30}

while true; do
    memory_usage=$(free --bytes --wide)
    total_bytes=$(grep 'Mem:' <<<"$memory_usage" | awk '{print $2}')
    free_bytes=$(grep 'Mem:' <<<"$memory_usage" | awk '{print $4}')

    free_percent=$((100 * free_bytes / total_bytes))
    text=""

    if [ $free_bytes -ge $((1024 * 1024 * 1024)) ]; then
        text="$((free_bytes / 1024 / 1024 / 1024)) GiB"
    fi

    tooltip="${free_percent}%"

    jq --unbuffered --compact-output <<EOF
{
"text":"${text}",
"tooltip":"${tooltip}",
"percentage":${free_percent}
}
EOF
    sleep $SLEEP_INTERVAL
done
