{
  config,
  lib,
  pkgs,
  ...
}:

{
  imports = [ ];

  environment.systemPackages = with pkgs; [
    powertop
  ];

  # amdgpu.abmlevel=3 :: Automatically reduce screen brightness but tweak colors to compensate for power reduction.
  # pcie_aspm=force pcie_aspm.policy=powersupersave :: Enable PCIe active state power management for power reduction.
  # nowatchdog :: Disable watchdog for power savings (related to disable_sp5100_watchdog above).
  # amd_pstate=passive :: Fully automated hardware pstate control.
  # amd_pstate=active :: Same as passive except we can set the energy performance preference (EPP) to suggest how much we prefer performance or energy efficiency.
  # amd_pstate=guided :: Same as passive except we can set upper and lower frequency bounds.
  # amdgpu.dcdebugmask=0x10 :: Allegedly disables Panel Replay from https://community.frame.work/t/tracking-freezing-arch-linux-amd/39495/32
  boot.kernelParams = [
    "amdgpu.abmlevel=3"
    "pcie_aspm=force"
    # "pcie_aspm.policy=powersupersave"
    "nowatchdog"
    # I don't see a measurable benefit from these two:
    # "cpufreq.default_governor=powersave"
    # "initcall_blacklist=cpufreq_gov_userspace_init"
  ];

  systemd.tmpfiles.rules = [
    "w- /sys/firmware/acpi/platform_profile - - - - low-power"
    "w- /sys/devices/system/cpu/cpufreq/policy0/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy1/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy2/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy3/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy4/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy5/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy6/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy7/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy8/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy9/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy10/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy11/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy12/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy13/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy14/energy_performance_preference - - - - power"
    "w- /sys/devices/system/cpu/cpufreq/policy15/energy_performance_preference - - - - power"
  ];

  boot.extraModprobeConfig = ''
    # Disable the hardware watchdog inside AMD 700 chipset series for power savings.
    blacklist sp5100_tco

    # Sound power-saving was causing chat notifications to be inaudible.
    # options snd_hda_intel power_save=1
  '';
}
