{
  config,
  lib,
  pkgs,
  ...
}:
{
  imports = [
    ./disk-config.nix
    ./distributed_build.nix
    ./hardware-configuration.nix
    ./optimized_build.nix
    ./power_management.nix
    ./wifi.nix
  ];

  # Generate with `head -c4 /dev/urandom | od -A none -t x4`
  networking.hostId = "47ee7d7c";

  networking.hostName = "quark"; # Define your hostname.

  time.timeZone = "America/New_York";
  i18n.defaultLocale = "en_US.UTF-8";

  me.optimizations.enable = true;
  me.secureBoot.enable = true;

  # Early KMS
  boot.initrd.kernelModules = [ "amdgpu" ];

  # Mount tmpfs at /tmp
  boot.tmp.useTmpfs = true;

  # Enable TRIM
  # services.fstrim.enable = lib.mkDefault true;

  me.alacritty.enable = true;
  me.ansible.enable = true;
  me.ares.enable = true;
  me.bluetooth.enable = true;
  me.chromecast.enable = true;
  me.chromium.enable = true;
  me.docker.enable = true;
  me.ecc.enable = true;
  me.emacs_flavor = "full";
  me.firefox.enable = true;
  me.flux.enable = true;
  me.gcloud.enable = true;
  me.git.config = ../../roles/git/files/gitconfig_home;
  me.gnuplot.enable = true;
  me.gpg.enable = true;
  me.graphical = true;
  me.graphics_card_type = "amd";
  me.kanshi.enable = false;
  me.kubernetes.enable = true;
  me.latex.enable = true;
  me.launch_keyboard.enable = true;
  me.lvfs.enable = true;
  me.media.enable = true;
  me.nix_index.enable = true;
  me.nix_worker.enable = true;
  me.pcsx2.enable = true;
  me.python.enable = true;
  me.qemu.enable = true;
  me.rust.enable = true;
  me.shikane.enable = true;
  me.sops.enable = true;
  me.sound.enable = true;
  me.steam.enable = true;
  me.steam_run_free.enable = true;
  me.sway.enable = true;
  me.tekton.enable = true;
  me.terraform.enable = true;
  me.thunderbolt.enable = true;
  me.vnc_client.enable = true;
  me.vscode.enable = true;
  me.wasm.enable = true;
  me.waybar.enable = true;
  me.wireguard.activated = [
    "drmario"
    "wgh"
    "colo"
  ];
  me.wireguard.deactivated = [ "wgf" ];
  me.zrepl.enable = true;
  me.zsh.enable = true;

  me.sm64ex.enable = true;
  me.shipwright.enable = true;
  me.ship2harkinian.enable = true;
}
